'use strict';

var fs = require('fs');
var path = require('path');

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

function isEmptyArray(arr) {
	return Array.isArray(arr) && arr.length === 0;
}

function isValid(metaValue) {
	return metaValue && !isEmptyArray(metaValue) && !isEmptyArray(Object.keys(metaValue));
}

function validateMatchPattern(urlPattern) {
	// eslint-disable-next-line no-useless-escape
	var pattern = /(\*|https?|file|ftp):\/\/(\*|\*\.[^\/\*]*|[^\/\*]*)\/(.*)/;

	return pattern.test(urlPattern);
}

function toMetaBlockString(key, value) {
	var padOffset = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 0;

	if (value) {
		return ('// @' + key.padEnd(padOffset) + ' ' + value).trim();
	} else {
		return '// @' + key;
	}
}

function processMetaValue(value) {
	if ((typeof value === 'undefined' ? 'undefined' : _typeof(value)) === 'object') {
		if (Array.isArray(value)) {
			return value.reduce(function (acc, val) {
				return acc.concat(val);
			}, []);
		} else {
			// key-value Object
			return Object.entries(value).reduce(function (acc, val) {
				acc.push(val.join(' '));
				return acc;
			}, []);
		}
	} else if (typeof value === 'string') {
		return [value];
	} else if (value) {
		// not falsy
		return [''];
	} else {
		// unknown
		return [];
	}
}

var index = function () {
	var options = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

	var opt = Object.assign({
		file: null,
		meta: null,
		// script manager : 'tampermonkey' | 'greasemonkey3' | 'greasemonkey4' | 'compatible'(default)
		manager: 'compatible',
		order: ['name', 'description', 'namespace', '...', 'grant'],
		version: null
	}, options);

	var meta = {
		name: 'New Userscript',
		namespace: 'FlandreDaisuki/rollup-plugin-userscript-metablock'
	};

	if (opt.file) {
		if (!fs.existsSync(opt.file)) {
			throw new Error('Metablock file not found: ' + opt.file);
		}

		var json = void 0;
		switch (path.extname(opt.file)) {
			case '.json':
				json = JSON.parse(fs.readFileSync(opt.file, 'utf8'));
				break;
			case '.js':
				json = require(opt.file);
				break;
		}

		Object.assign(meta, json);
	}

	if (opt.meta) {
		Object.assign(meta, opt.meta);
	}

	if (opt.version) {
		meta.version = opt.version;
	}

	// always grant
	if (!isValid(meta.grant)) {
		meta.grant = 'none';
	}

	if (meta.match) {
		if (Array.isArray(meta.match)) {
			var _iteratorNormalCompletion = true;
			var _didIteratorError = false;
			var _iteratorError = undefined;

			try {
				for (var _iterator = meta.match[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
					var urlPattern = _step.value;

					if (!validateMatchPattern(urlPattern)) {
						throw new Error('Invalid match pattern: ' + urlPattern + '.');
					}
				}
			} catch (err) {
				_didIteratorError = true;
				_iteratorError = err;
			} finally {
				try {
					if (!_iteratorNormalCompletion && _iterator.return) {
						_iterator.return();
					}
				} finally {
					if (_didIteratorError) {
						throw _iteratorError;
					}
				}
			}
		} else {
			if (!validateMatchPattern(meta.match)) {
				throw new Error('Invalid match pattern: ' + meta.match + '.');
			}
		}
	}

	// if no include rule, greasemonkey assume `@include *` but tampermonkey don't.
	if (!isValid(meta.include) && !isValid(meta.match) && opt.manager === 'compatible') {
		meta.include = '*';
	}

	// process key-value form name / description
	var _iteratorNormalCompletion2 = true;
	var _didIteratorError2 = false;
	var _iteratorError2 = undefined;

	try {
		for (var _iterator2 = Object.entries(meta)[Symbol.iterator](), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
			var _ref = _step2.value;

			var _ref2 = _slicedToArray(_ref, 2);

			var metaKey = _ref2[0];
			var metaValue = _ref2[1];

			if ((metaKey === 'name' || metaKey === 'description') && (typeof metaValue === 'undefined' ? 'undefined' : _typeof(metaValue)) === 'object') {
				if (!metaValue.default) {
					throw new Error('The key-value ' + metaKey + ' need to provide default attribute.');
				}

				delete meta[metaKey];

				var _iteratorNormalCompletion7 = true;
				var _didIteratorError7 = false;
				var _iteratorError7 = undefined;

				try {
					for (var _iterator7 = Object.entries(metaValue)[Symbol.iterator](), _step7; !(_iteratorNormalCompletion7 = (_step7 = _iterator7.next()).done); _iteratorNormalCompletion7 = true) {
						var _ref5 = _step7.value;

						var _ref6 = _slicedToArray(_ref5, 2);

						var lang = _ref6[0];
						var str = _ref6[1];

						if (lang === 'default') {
							meta[metaKey] = str;
						} else {
							meta[metaKey + ':' + lang] = str;
						}
					}
				} catch (err) {
					_didIteratorError7 = true;
					_iteratorError7 = err;
				} finally {
					try {
						if (!_iteratorNormalCompletion7 && _iterator7.return) {
							_iterator7.return();
						}
					} finally {
						if (_didIteratorError7) {
							throw _iteratorError7;
						}
					}
				}
			}
		}
	} catch (err) {
		_didIteratorError2 = true;
		_iteratorError2 = err;
	} finally {
		try {
			if (!_iteratorNormalCompletion2 && _iterator2.return) {
				_iterator2.return();
			}
		} finally {
			if (_didIteratorError2) {
				throw _iteratorError2;
			}
		}
	}

	Set.prototype.difference = function (setB) {
		var _iteratorNormalCompletion3 = true;
		var _didIteratorError3 = false;
		var _iteratorError3 = undefined;

		try {
			for (var _iterator3 = setB[Symbol.iterator](), _step3; !(_iteratorNormalCompletion3 = (_step3 = _iterator3.next()).done); _iteratorNormalCompletion3 = true) {
				var elem = _step3.value;

				this.delete(elem);
			}
		} catch (err) {
			_didIteratorError3 = true;
			_iteratorError3 = err;
		} finally {
			try {
				if (!_iteratorNormalCompletion3 && _iterator3.return) {
					_iterator3.return();
				}
			} finally {
				if (_didIteratorError3) {
					throw _iteratorError3;
				}
			}
		}
	};

	// reorder metakeys
	var metaKeySet = new Set(Object.keys(meta));
	var pre = [],
	    post = [];

	var isToPre = true;

	var _loop = function _loop(oKey) {
		if (oKey === '...') {
			isToPre = false;
		} else {
			var keyIncludeOKey = [].concat(_toConsumableArray(metaKeySet)).filter(function (k) {
				return k.includes(oKey);
			});
			keyIncludeOKey.sort();
			if (isToPre) {
				pre.push.apply(pre, _toConsumableArray(keyIncludeOKey));
			} else {
				post.push.apply(post, _toConsumableArray(keyIncludeOKey));
			}
			metaKeySet.difference(keyIncludeOKey);
		}
	};

	var _iteratorNormalCompletion4 = true;
	var _didIteratorError4 = false;
	var _iteratorError4 = undefined;

	try {
		for (var _iterator4 = opt.order[Symbol.iterator](), _step4; !(_iteratorNormalCompletion4 = (_step4 = _iterator4.next()).done); _iteratorNormalCompletion4 = true) {
			var oKey = _step4.value;

			_loop(oKey);
		}
	} catch (err) {
		_didIteratorError4 = true;
		_iteratorError4 = err;
	} finally {
		try {
			if (!_iteratorNormalCompletion4 && _iterator4.return) {
				_iterator4.return();
			}
		} finally {
			if (_didIteratorError4) {
				throw _iteratorError4;
			}
		}
	}

	var orderedMetaKeys = [].concat(_toConsumableArray(pre), _toConsumableArray(metaKeySet), _toConsumableArray(post));
	var orderedMeta = {};
	var _iteratorNormalCompletion5 = true;
	var _didIteratorError5 = false;
	var _iteratorError5 = undefined;

	try {
		for (var _iterator5 = orderedMetaKeys[Symbol.iterator](), _step5; !(_iteratorNormalCompletion5 = (_step5 = _iterator5.next()).done); _iteratorNormalCompletion5 = true) {
			var omkey = _step5.value;

			orderedMeta[omkey] = meta[omkey];
		}
	} catch (err) {
		_didIteratorError5 = true;
		_iteratorError5 = err;
	} finally {
		try {
			if (!_iteratorNormalCompletion5 && _iterator5.return) {
				_iterator5.return();
			}
		} finally {
			if (_didIteratorError5) {
				throw _iteratorError5;
			}
		}
	}

	delete Set.prototype.difference;

	// process all metakeys
	var metaOutputList = [];
	var maxMetaKeyLength = Object.keys(orderedMeta).reduce(function (prev, val) {
		return Math.max(prev, val.length);
	}, 0);

	var _iteratorNormalCompletion6 = true;
	var _didIteratorError6 = false;
	var _iteratorError6 = undefined;

	try {
		for (var _iterator6 = Object.entries(orderedMeta)[Symbol.iterator](), _step6; !(_iteratorNormalCompletion6 = (_step6 = _iterator6.next()).done); _iteratorNormalCompletion6 = true) {
			var _ref3 = _step6.value;

			var _ref4 = _slicedToArray(_ref3, 2);

			var _metaKey = _ref4[0];
			var _metaValue = _ref4[1];
			var _iteratorNormalCompletion8 = true;
			var _didIteratorError8 = false;
			var _iteratorError8 = undefined;

			try {
				for (var _iterator8 = processMetaValue(_metaValue)[Symbol.iterator](), _step8; !(_iteratorNormalCompletion8 = (_step8 = _iterator8.next()).done); _iteratorNormalCompletion8 = true) {
					var processed = _step8.value;

					metaOutputList.push(toMetaBlockString(_metaKey, processed, maxMetaKeyLength));
				}
			} catch (err) {
				_didIteratorError8 = true;
				_iteratorError8 = err;
			} finally {
				try {
					if (!_iteratorNormalCompletion8 && _iterator8.return) {
						_iterator8.return();
					}
				} finally {
					if (_didIteratorError8) {
						throw _iteratorError8;
					}
				}
			}
		}
	} catch (err) {
		_didIteratorError6 = true;
		_iteratorError6 = err;
	} finally {
		try {
			if (!_iteratorNormalCompletion6 && _iterator6.return) {
				_iterator6.return();
			}
		} finally {
			if (_didIteratorError6) {
				throw _iteratorError6;
			}
		}
	}

	metaOutputList.unshift('// ==UserScript==');
	metaOutputList.push('// ==/UserScript==');
	var metaOutputString = metaOutputList.join('\n') + '\n\n';

	return {
		name: 'rollup-plugin-userscript-metablock',
		transformBundle: function transformBundle(source) {
			return metaOutputString + source;
		}
	};
};

module.exports = index;
